/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher.api.server;

import java.io.File;
import java.util.List;
import java.util.Properties;

import org.ow2.petals.launcher.api.server.conf.ConfigurationProperties;
import org.ow2.petals.launcher.api.server.exception.PetalsServerException;
import org.ow2.petals.launcher.api.service.ServiceEndpoint;
import org.ow2.petals.topology.generated.Topology;

/**
 * <p>
 * The public API of the Petals container.
 * </p>
 * <p>
 * The configuration of the logging system is provided through a file defined by
 * the system property
 * {@value ConfigurationProperties#LOGGING_SYSTEM_CONF_FILE_PROPERTY_NAME} . If
 * the previous property is not defined, the default configuration file will be
 * {@value ConfigurationProperties#DEFAULT_LOGGER_FILENAME}. If it is not
 * available on the classpath, an error will occurs.
 * </p>
 * <p>
 * Each property value of the logging system configuration file can contain
 * reference to other properties using a placeholder as:<br/>
 * <code>org.ow2.petals.log.handler.PetalsFileHandler.basedir=${petals.log.dir}</code>
 * <br/>
 * Available properties to use as placeholder are:
 * <ul>
 * <li>the system properties including all properties defined on the java
 * command line using the flag '<code>-D</code>',</li>
 * <li>the properties of the container configuration file (
 * <code>server.properties</code>).</li>
 * </ul>
 * 
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 */
public interface PetalsServer {

    public static final String COMPONENT_LOGGER_NAME = "Petals.Server";

    /**
     * Initialize the PEtALS environment
     * 
     * @param serverLocalProperties
     *            Configuration local properties of the current Petals container
     *            (ie. the content of the file '<code>server.properties</code>'.
     * @param topology
     *            The topology configuration. Can not be <code>null</code>.
     * @param dataRootPath
     *            The root path of the Petals data
     * @throws PetalsServerException
     *             Raised if any error during initialization
     */
    void init(final Properties serverLocalProperties, final Topology topology,
            final File dataRootPath) throws PetalsServerException;

    /**
     * Start the PEtALS container
     * 
     * @throws PetalsServerException
     *             Raised if any error during starting
     */
    void start() throws PetalsServerException;

    /**
     * Stop the PEtALS container.
     * 
     * @throws PetalsServerException
     *             Raised if any error during stopping
     */
    void stop() throws PetalsServerException;

    /**
     * Browse the content of the PEtALS JNDI directory.
     * 
     * @return A Content in a String form
     * @throws PetalsServerException
     *             Raised if any error during browsing
     */
    String browseJNDI() throws PetalsServerException;

    /**
     * 
     * @param if true, get all the endpoints from all the containers I can talk
     *        with. If false, get all the endpoints I know locally
     * @return
     */
    List<ServiceEndpoint> getServiceEndpoints(boolean global) throws PetalsServerException;

    /**
     * Get the container configuration. Must be called once the PEtALS container
     * is started.
     * 
     * @return The container configuration in a String form
     * @throws PetalsServerException
     *             If the container configuration is not defined
     */
    String getContainerConfiguration() throws PetalsServerException;

    /**
     * @return <code>true</code> if the Petals container is starting, started; <code>false</code> if the container is
     *         stopping or stopped
     */
    public boolean isRunning();

}
