/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.launcher.api.server;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;

import org.ow2.petals.launcher.api.server.exception.DuplicatedServiceException;
import org.ow2.petals.launcher.api.server.exception.MissingServiceException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * The Petals server Factory
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public final class PetalsServerFactory {

    /**
     * Name of the property used to select the implementation of Petals server
     * API.
     */
    public final static String PROPERTY_NAME = PetalsServer.class.getName();

    private final static Object INITIALIZATION_LOCK = new Object();

    private static PetalsServer API_INSTANCE = null;

    /**
     * <p>
     * Get the Petals server instance using an implementation according to the
     * following rule:
     * <ul>
     * <li>first, if a property with the service name is set, it is used to look
     * for the service implementation,</li>
     * <li>otherwise the service implementation is looking for in the current
     * classloader.</li>
     * </ul>
     * </p>
     * <p>
     * On the first call, the Petals server API instance is created.
     * </p>
     * 
     * @throws DuplicatedServiceException
     *             The service to look for is duplicated.
     * @throws MissingServiceException
     *             Unable to find the service.
     */
    public static final PetalsServer getInstance() throws DuplicatedServiceException,
            MissingServiceException {
        synchronized (INITIALIZATION_LOCK) {
            if (API_INSTANCE == null) {
                final String overridenImplementation = System
                        .getProperty(PetalsServerFactory.PROPERTY_NAME);
                if (overridenImplementation == null || overridenImplementation.trim().isEmpty()) {
                    final ServiceLoader<PetalsServer> serviceLoader = ServiceLoader
                            .load(PetalsServer.class);
                    final Iterator<PetalsServer> servicesIterator = serviceLoader.iterator();
                    try {
                        if (servicesIterator.hasNext()) {
                            final PetalsServer serviceLoaded = servicesIterator.next();
                            if (servicesIterator.hasNext()) {
                                throw new DuplicatedServiceException(PetalsServer.class);
                            } else {
                                API_INSTANCE = serviceLoaded;
                            }
                        } else {
                            throw new MissingServiceException(PetalsServer.class);
                        }
                    } catch (final ServiceConfigurationError e) {
                        throw new MissingServiceException(PetalsServer.class, e);
                    }
                } else {
                    try {
                        API_INSTANCE = ReflectionHelper.newInstance(overridenImplementation, null);
                    } catch (final InvocationTargetException e) {
                        throw new MissingServiceException(PetalsServer.class, e);
                    }
                }
            }

            return API_INSTANCE;
        }
    }

    /**
     * Free resources allocated by the Petals server factgory instance
     */
    public static final void close() {
        synchronized (INITIALIZATION_LOCK) {
            API_INSTANCE = null;
        }
    }

}
