/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.kernel.api.server;

import static junit.framework.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;

import junit.framework.Assert;

import org.junit.Test;
import org.ow2.petals.kernel.api.server.utest.PetalsServerUTest;
import org.ow2.petals.launcher.api.server.PetalsServer;
import org.ow2.petals.launcher.api.server.PetalsServerFactory;
import org.ow2.petals.launcher.api.server.exception.DuplicatedServiceException;
import org.ow2.petals.launcher.api.server.exception.MissingServiceException;
import org.ow2.petals.launcher.api.server.exception.PetalsServerException;
import org.ow2.petals.launcher.api.service.ServiceEndpoint;
import org.ow2.petals.topology.generated.Topology;

/**
 * Unit tests of the Petals server factory: {@link PetalsServerFactory}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PetalsServerFactoryTest {

    /**
     * Dummy implementation of {@link PetalsServer} for duplication test.
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    public class PetalsServerDummy implements PetalsServer {

        @Override
        public void init(Properties serverLocalProperties, Topology topology, File dataRootPath)
                throws PetalsServerException {
            // NOP
        }

        @Override
        public void start() throws PetalsServerException {
            // NOP
        }

        @Override
        public void stop() throws PetalsServerException {
            // NOP
        }

        @Override
        public String browseJNDI() throws PetalsServerException {
            return null;
        }

        @Override
        public List<ServiceEndpoint> getServiceEndpoints(boolean global)
                throws PetalsServerException {
            return null;
        }

        @Override
        public String getContainerConfiguration() throws PetalsServerException {
            return null;
        }

        @Override
        public boolean isRunning() {
            return false;
        }

    }

    /**
     * <p>
     * Check the loading of the Petals server implementation when only one
     * implementation is registered through the classpath.
     * </p>
     * <p>
     * Expected result: The implementation available on the classpath is laoded.
     * </p>
     */
    @Test
    public void testGetInstance_00() throws DuplicatedServiceException,
            MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsServer.class.getName())) {
                    return super.getResources("services/utest/" + PetalsServer.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final PetalsServer petalsServer = PetalsServerFactory.getInstance();
            Assert.assertTrue(petalsServer instanceof PetalsServerUTest);
        } finally {
            PetalsServerFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals server implementation when no
     * implementation is registered through the classpath and the property is
     * not set.
     * </p>
     * <p>
     * Expected result: The exception {@link MissingServiceException} is thrown.
     * </p>
     */
    @Test(expected = MissingServiceException.class)
    public void testGetInstance_01() throws DuplicatedServiceException,
            MissingServiceException {

        try {
            PetalsServerFactory.getInstance();
            fail("Exception " + MissingServiceException.class.getName() + " is not thrown !");
        } finally {
            PetalsServerFactory.close();
        }
    }

    /**
     * <p>
     * Check the loading of the Petals server implementation when no
     * implementation is registered through the classpath and the property is
     * set.
     * </p>
     * <p>
     * Expected result: The implementation defined by the property is loaded.
     * </p>
     */
    @Test
    public void testGetInstance_02() throws DuplicatedServiceException,
            MissingServiceException {

        System.setProperty(PetalsServerFactory.PROPERTY_NAME,
                PetalsServerUTest.class.getName());
        try {
            final PetalsServer petalsServer = PetalsServerFactory.getInstance();
            Assert.assertTrue(petalsServer instanceof PetalsServerUTest);
        } finally {
            PetalsServerFactory.close();
            System.clearProperty(PetalsServerFactory.PROPERTY_NAME);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals server implementation when several
     * implementations are registered through the classpath.
     * </p>
     * <p>
     * Expected result: The exception {@link DuplicatedServiceException} is
     * thrown.
     * </p>
     */
    @Test(expected = DuplicatedServiceException.class)
    public void testGetInstance_03() throws DuplicatedServiceException,
            MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsServer.class.getName())) {
                    return super.getResources("services/duplicated/"
 + PetalsServer.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            PetalsServerFactory.getInstance();
            fail("Exception " + DuplicatedServiceException.class.getName() + " is not thrown !");
        } finally {
            PetalsServerFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals server implementation when several
     * implementations are registered through the classpath, and the property is
     * set to one of the implementation registered through the classpath.
     * </p>
     * <p>
     * Expected result: The implementation defined by the property is loaded.
     * </p>
     */
    @Test
    public void testGetInstance_04() throws DuplicatedServiceException, MissingServiceException {

        System.setProperty(PetalsServerFactory.PROPERTY_NAME,
                PetalsServerUTest.class.getName());

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsServer.class.getName())) {
                    return super.getResources("services/duplicated/"
 + PetalsServer.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            final PetalsServer petalsServer = PetalsServerFactory.getInstance();
            Assert.assertTrue(petalsServer instanceof PetalsServerUTest);
        } finally {
            PetalsServerFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
            System.clearProperty(PetalsServerFactory.PROPERTY_NAME);
        }
    }

    /**
     * <p>
     * Check the loading of the Petals server implementation when only one
     * implementation is registered through the classpath, but the class does
     * not exist.
     * </p>
     * <p>
     * Expected result: The exception {@link MissingServiceException} is thrown.
     * </p>
     */
    @Test(expected = MissingServiceException.class)
    public void testGetInstance_05() throws DuplicatedServiceException, MissingServiceException {

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new ClassLoader(oldClassloader) {

            @Override
            public Enumeration<URL> getResources(final String name) throws IOException {
                if (name.equals("META-INF/services/" + PetalsServer.class.getName())) {
                    return super.getResources("services/class-doesnot-exist/"
                            + PetalsServer.class.getName());
                } else {
                    return super.getResources(name);
                }
            }
        });

        try {
            PetalsServerFactory.getInstance();
            fail("Exception " + MissingServiceException.class.getName() + " is not thrown !");
        } finally {
            PetalsServerFactory.close();
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }
}
